package com.namasoft.common.utils.translation;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map.Entry;

import com.namasoft.common.constants.Language;
import com.namasoft.common.utilities.ObjectChecker;

public class MessageTranslatorFromDB extends MessageTranslatorBase
{
	private HashMap<String, String> arabic = new HashMap<String, String>();
	private HashMap<String, String> english = new HashMap<String, String>();
	private HashMap<String, String> replacements = new HashMap<String, String>();
	private long order;

	public MessageTranslatorFromDB(HashMap<String, String> arabic, HashMap<String, String> english, HashMap<String, String> replacements, long order)
	{
		super();
		this.arabic = arabic;
		this.english = english;
		this.replacements = replacements;
		this.order = order;
	}

	public MessageTranslatorFromDB()
	{
	}

	@Override
	public List<String> getKeys()
	{
		HashSet<String> keys = new HashSet<String>();
		keys.addAll(arabic.keySet());
		keys.addAll(english.keySet());
		return new ArrayList<String>(keys);
	}

	@Override
	public long getOrder()
	{
		return order;
	}

	@Override
	public void processRecursions()
	{
		processLang(arabic, Language.Arabic);
		processLang(english, Language.English);
	}

	@Override
	public void processReplacements(List<MessageTranslator> translators)
	{
		for (Entry<String, String> replacement : replacements.entrySet())
		{
			for (MessageTranslator translator : translators)
			{
				translator.replace(replacement.getKey(), replacement.getValue());
			}
		}
	}

	@Override
	public String translateMessage(Language lang, String key)
	{
		return map(lang).get(key);
	}

	private HashMap<String, String> map(Language lang)
	{
		return lang == Language.English ? english : arabic;
	}

	@Override
	public void replace(String key, String value)
	{
		for (Entry<String, String> entry : arabic.entrySet())
		{
			entry.setValue(ObjectChecker.toStringOrEmpty(entry.getValue())
					.replace(ObjectChecker.toStringOrEmpty(key), ObjectChecker.toStringOrEmpty(value)));
		}
		for (Entry<String, String> entry : english.entrySet())
		{
			entry.setValue(ObjectChecker.toStringOrEmpty(entry.getValue())
					.replace(ObjectChecker.toStringOrEmpty(key), ObjectChecker.toStringOrEmpty(value)));
		}
	}
}
